library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
Library UNISIM;
use UNISIM.vcomponents.all;

entity top_level is

Port (CLK								: IN  STD_LOGIC;
		mpmc_0_SDRAM_Clk_pin 		: OUT std_logic;
		mpmc_0_SDRAM_CE_pin  		: OUT std_logic;
		mpmc_0_SDRAM_CS_n_pin 		: OUT std_logic;
		mpmc_0_SDRAM_RAS_n_pin 		: OUT std_logic;
		mpmc_0_SDRAM_CAS_n_pin 		: OUT std_logic;
		mpmc_0_SDRAM_WE_n_pin 		: OUT std_logic;
		mpmc_0_SDRAM_BankAddr_pin 	: OUT std_logic_vector(1 downto 0);
		mpmc_0_SDRAM_Addr_pin 		: OUT std_logic_vector(12 downto 0);
		mpmc_0_SDRAM_DQ 				: INOUT std_logic_vector(15 downto 0);
		mpmc_0_SDRAM_DM_pin 			: OUT std_logic_vector(1 downto 0);
		J20								: IN 	STD_LOGIC_VECTOR(134 downto 0);
		J16								: IN 	STD_LOGIC_VECTOR(7 downto 0);
		J17								: IN 	STD_LOGIC_VECTOR(7 downto 0);
		J18								: IN 	STD_LOGIC_VECTOR(7 downto 0);
		BTN	  	    					: IN 	STD_LOGIC_VECTOR(1 downto 0);
		RX		  	    					: IN  STD_LOGIC;
		TX		  	    					: OUT STD_LOGIC;
		LED								: OUT STD_LOGIC_VECTOR(3 downto 0);
		SDA   	    					: INOUT  STD_LOGIC;
		SCL	       					: INOUT  STD_LOGIC;
		SPI_nSS_pin 					: OUT  STD_LOGIC;
      SPI_MOSI_pin 					: OUT  STD_LOGIC;
		TILE1_REFCLK_PAD_N_IN      : IN   std_logic;
		TILE1_REFCLK_PAD_P_IN      : IN   std_logic;
		RXN_IN                     : IN   std_logic_vector(5 downto 0);
		RXP_IN                     : IN   std_logic_vector(5 downto 0);
		TXN_OUT                    : OUT  std_logic_vector(5 downto 0);
		TXP_OUT                    : OUT  std_logic_vector(5 downto 0));
		
end top_level;

architecture Behavioral of top_level is
----------------------------------------------
	COMPONENT LOOP_TEST_TOP
	PORT(
		TILE1_REFCLK_PAD_N_IN : IN std_logic;
		TILE1_REFCLK_PAD_P_IN : IN std_logic;
		GTXRESET_IN : IN std_logic;
		POWERDOWN	: IN std_logic;
		RXN_IN : IN std_logic_vector(5 downto 0);
		RXP_IN : IN std_logic_vector(5 downto 0);          
		TILE0_1_2_PLLLKDET_OUT : OUT std_logic;
		TRACK_DATA_OUT : OUT std_logic_vector(2 downto 0);
		TXN_OUT : OUT std_logic_vector(5 downto 0);
		TXP_OUT : OUT std_logic_vector(5 downto 0)
		);
	END COMPONENT;

	COMPONENT system
	PORT(
		fpga_0_RS232_RX_pin : IN std_logic;
		fpga_0_clk_1_sys_clk_pin : IN std_logic;
		fpga_0_rst_1_sys_rst_pin : IN std_logic;    
		mpmc_0_SDRAM_DQ : INOUT std_logic_vector(15 downto 0);      
		fpga_0_RS232_TX_pin : OUT std_logic;
		mpmc_0_SDRAM_Clk_pin : OUT std_logic;
		mpmc_0_SDRAM_CE_pin : OUT std_logic;
		mpmc_0_SDRAM_CS_n_pin : OUT std_logic;
		mpmc_0_SDRAM_RAS_n_pin : OUT std_logic;
		mpmc_0_SDRAM_CAS_n_pin : OUT std_logic;
		mpmc_0_SDRAM_WE_n_pin : OUT std_logic;
		mpmc_0_SDRAM_BankAddr_pin : OUT std_logic_vector(1 downto 0);
		mpmc_0_SDRAM_Addr_pin : OUT std_logic_vector(12 downto 0);
		mpmc_0_SDRAM_DM_pin : OUT std_logic_vector(1 downto 0);
		mpmc_0_MPMC_InitDone_pin : OUT std_logic
		);
	END COMPONENT;
	
	attribute box_type : string;
	attribute box_type of system : component is "user_black_box";
	
	component uart_tx
	port (
    i_clk       : in  std_logic;
    i_tx_dv     : in  std_logic;
    i_tx_byte   : in  std_logic_vector(7 downto 0);
    o_tx_active : out std_logic;
    o_tx_serial : out std_logic;
    o_tx_done   : out std_logic
    );
	END component;

	component uart_rx
	port (
    i_clk       : in  std_logic;
    i_rx_serial : in  std_logic;
    o_rx_dv     : out std_logic;
    o_rx_byte   : out std_logic_vector(7 downto 0)
    );
	END component;

	COMPONENT i2c_master
	PORT(
		clk 			: IN std_logic;
		reset_n 		: IN std_logic;
		ena 			: IN std_logic;
		addr 			: IN std_logic_vector(6 downto 0);
		rw 			: IN std_logic;
		data_wr 		: IN std_logic_vector(7 downto 0);    
		sda 			: INOUT std_logic;
		scl 			: INOUT std_logic;      
		busy 			: OUT std_logic;
		data_rd 		: OUT std_logic_vector(7 downto 0);
		ack_error	: OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT debounce
	PORT(
		clk : IN std_logic;
		button : IN std_logic;          
		result : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT DBounce
	PORT(
		clk : IN std_logic;
		nreset : IN std_logic;
		button_in : IN std_logic;          
		DB_out : OUT std_logic
		);
	END COMPONENT;
---------------------------------------------
signal i_o					:STD_LOGIC_VECTOR(159 downto 0);
signal timer				:STD_LOGIC_VECTOR(35 downto 0);
signal count_d			 	:STD_LOGIC_VECTOR(24 DOWNTO 0);
signal count_pwr  		:STD_LOGIC_VECTOR(26 DOWNTO 0);
signal rx_data,i2c_data_rd,i2c_data_wr,o_rx_data,io_sum,i2c_data_write,i2c_data_wrr :STD_LOGIC_VECTOR(7 DOWNTO 0);
signal mode					:STD_LOGIC_VECTOR(4 downto 0);
signal led0,led1,led2,led3,led4,led5	:STD_LOGIC_VECTOR(3 downto 0);
signal rx_dv,s_1,s_2,s_3,s_4,s_5,tx_dv,ena_i2c_write,power_up,reset,tx_dv_en,lock_rx,ram_mode,prog_mode,io_btn,MPMC_InitDone,SPI_MISO,
io_test_mode,GTX_mode,spi_mode,tx_serial,five_sec,ena_i2c,ena_i2c_read,stop_iic,i2c_busy,rw_i2c,microblaze_tx,SPI_SCK,
rst_i2c,count_start,start_iic,rx_serial,CE,sec_135,ten_min,rx_data_switch,ok,SPI_ID_OK,SPI_test_result,microblaze_reset,
chk_byte1_bit1,chk_byte1_bit2,chk_byte1_bit3,chk_byte1_bit4,chk_byte1_bit5,chk_byte1_bit6,chk_byte1_bit7,chk_byte1_bit8,chk_byte2_bit1,chk_byte2_bit2,chk_byte2_bit3,chk_byte2_bit4,
chk_byte2_bit5,chk_byte2_bit6,chk_byte2_bit7,chk_byte2_bit8,chk_byte3_bit1,chk_byte3_bit2,chk_byte3_bit3,chk_byte3_bit4,chk_byte3_bit5,chk_byte3_bit6,chk_byte3_bit7,chk_byte3_bit8 :STD_LOGIC;
---------------------------------------------
begin

	STARTUP_VIRTEX5_inst : STARTUP_VIRTEX5
   port map (
      CFGCLK => open,     		-- Config logic clock 1-bit output
      CFGMCLK => open,    		-- Config internal osc clock 1-bit output
      DINSPI => SPI_MISO, 		-- DIN SPI PROM access 1-bit output
      EOS => open,        		-- End of Startup 1-bit output
		TCKSPI => open,
      CLK => open,     		   -- Clock input for start-up sequence
      GSR => '0',  		 		-- Global Set/Reset input (GSR cannot be used for the port name)
      GTS => '0',  				-- Global 3-state input (GTS cannot be used for the port name)
      USRCCLKO => SPI_SCK,   	-- User CCLK 1-bit input
      USRCCLKTS => '0', 		-- User CCLK 3-state, 1-bit input
      USRDONEO => open,  		-- User Done 1-bit input
      USRDONETS => open  		-- User Done 3-state, 1-bit input
   );
	
	Inst_LOOP_TEST_TOP: LOOP_TEST_TOP PORT MAP(
		TILE1_REFCLK_PAD_N_IN => TILE1_REFCLK_PAD_N_IN,
		TILE1_REFCLK_PAD_P_IN => TILE1_REFCLK_PAD_P_IN,
		GTXRESET_IN => reset,
		POWERDOWN => not GTX_mode,
		TILE0_1_2_PLLLKDET_OUT => LED4(0),
		TRACK_DATA_OUT => LED4(3 downto 1),
		RXN_IN => RXN_IN,
		RXP_IN => RXP_IN,
		TXN_OUT => TXN_OUT,
		TXP_OUT => TXP_OUT
	);

	Inst_system: system PORT MAP(
		fpga_0_RS232_RX_pin => '1',
		fpga_0_RS232_TX_pin => microblaze_tx,
		fpga_0_clk_1_sys_clk_pin => CLK,
		fpga_0_rst_1_sys_rst_pin => microblaze_reset,
      mpmc_0_SDRAM_Clk_pin => mpmc_0_SDRAM_Clk_pin,
      mpmc_0_SDRAM_CE_pin => mpmc_0_SDRAM_CE_pin,
      mpmc_0_SDRAM_CS_n_pin => mpmc_0_SDRAM_CS_n_pin,
      mpmc_0_SDRAM_RAS_n_pin => mpmc_0_SDRAM_RAS_n_pin,
      mpmc_0_SDRAM_CAS_n_pin => mpmc_0_SDRAM_CAS_n_pin,
      mpmc_0_SDRAM_WE_n_pin => mpmc_0_SDRAM_WE_n_pin,
      mpmc_0_SDRAM_BankAddr_pin => mpmc_0_SDRAM_BankAddr_pin,
      mpmc_0_SDRAM_Addr_pin => mpmc_0_SDRAM_Addr_pin,
      mpmc_0_SDRAM_DQ => mpmc_0_SDRAM_DQ,
      mpmc_0_SDRAM_DM_pin => mpmc_0_SDRAM_DM_pin,
		mpmc_0_MPMC_InitDone_pin => MPMC_InitDone
	);

	uarttx: uart_tx
	port map(	i_clk			=> clk,
					i_tx_dv		=> tx_dv,
					i_tx_byte	=> i2c_data_rd,
					o_tx_active => open,
					o_tx_done 	=> open,
					o_tx_serial	=> tx_serial);
					
	uartrx: uart_rx
	port map(	i_clk			=> clk,
					i_rx_serial => rx_serial,
					o_rx_dv 		=> rx_dv,
					o_rx_byte	=> o_rx_data);


	Inst_i2c_master: i2c_master PORT MAP(
		clk => clk,
		reset_n => rst_i2c,
		ena => ena_i2c,
		addr => "1010000", -- AT24C256 EEPROM ADDRESS
		rw => rw_i2c,
		data_wr => i2c_data_write,
		busy => i2c_busy,
		data_rd => i2c_data_rd,
		ack_error => open,
		sda => sda,
		scl => scl
	);

	Inst_debounce: debounce PORT MAP(
		clk => clk,
		button => btn(0),
		result => reset
	);
	
	Inst_DBounce: DBounce PORT MAP(
		clk => clk,
		nreset => reset,
		button_in => btn(1),
		DB_out => io_btn
	);

---------------------------------------------------------
	
	process(clk,reset)
	begin
	  if (reset='0') then
    count_pwr <= (others => '0');
  elsif (clk'event and clk = '1') then
      if (CE='0') then
			count_pwr <= count_pwr + '1';
		end if;
		end if;
	end process;
	
CE <= '1' when count_pwr= X"5F5E101" else '0';
power_up <= '1' when (count_pwr= X"3FFFE" or count_pwr= X"5F5E100") else '0';
rx_data_switch  <= power_up or five_sec or sec_135 or ten_min or not reset;

process(rx_data_switch,rx_dv,o_rx_data)
begin
  if (rx_data_switch = '1') then
      rx_data <= "00000000";
  elsif (rx_dv = '1') then
		rx_data <= o_rx_data;
  end if;
end process;

tx_dv <= not i2c_busy and tx_dv_en;
	
s_1 <= '1' when o_rx_data= "00110001" and rx_dv='1' else '0';
s_2 <= '1' when o_rx_data= "00110010" and rx_dv='1'  else '0';
s_3 <= '1' when o_rx_data= "00110011" and rx_dv='1'  else '0';
s_4 <= '1' when o_rx_data= "00110100" and rx_dv='1'  else '0';
s_5 <= '1' when o_rx_data= "00110101" and rx_dv='1'  else '0';

	process(clk,rx_dv,reset)
	begin
	  if ((rx_dv='1') or (reset='0')) then
    timer <= (others => '0');
  elsif (clk'event and clk = '1') then 
			timer <= timer + '1';
		end if;
	end process;

five_sec <= '1' when timer= X"EE6B280"   and spi_mode = '1' else '0';
sec_135 <= '1' when timer= X"19254D380" and ram_mode = '1' else '0';
ten_min  <= '1' when timer= X"6FC23AC00" and ((io_test_mode = '1') or (GTX_mode = '1')) else '0';

process(s_1,sec_135,power_up,reset)
begin
  if ((sec_135 = '1') or (power_up = '1') or (reset='0')) then
      ram_mode <= '0';
  elsif (s_1 = '1') then
		ram_mode <= '1';
  end if;
end process;

process(s_2,power_up,reset)
begin
  if ((power_up = '1') or (reset='0')) then
      prog_mode <= '0';
  elsif (s_2 = '1') then
		prog_mode <= '1';
  end if;
end process;

process(s_3,ten_min,power_up,reset)
begin
  if ((ten_min = '1') or (power_up = '1') or (reset='0')) then
      io_test_mode <= '0';
  elsif (s_3 = '1') then
		io_test_mode <= '1';
  end if;
end process;

process(s_4,ten_min,power_up,reset)
begin
  if ((ten_min = '1') or (power_up = '1') or (reset='0')) then
      GTX_mode <= '0';
  elsif (s_4 = '1') then
		GTX_mode <= '1';
  end if;
end process;

process(s_5,five_sec,power_up,reset)
begin
  if ((five_sec = '1') or (power_up = '1') or (reset='0')) then
      spi_mode <= '0';
  elsif (s_5 = '1') then
		spi_mode <= '1';
  end if;
end process;

mode(0) <= ram_mode;
mode(1) <= prog_mode;
mode(2) <= io_test_mode;
mode(3) <= GTX_mode;
mode(4) <= spi_mode;

process(mode,led1,led2,led3,led4,led0,led5)
begin
	   case mode is
		when "00001" => led<=led1;
      when "00010" => led<=led2;
		when "00100" => led<=led3;
		when "01000" => led<=led4;
		when "10000" => led<=led5;
      when others => led<=led0;
   end case;
end process;

lock_rx <= ram_mode or prog_mode or io_test_mode or GTX_mode or spi_mode or not ce;
rx_serial <= RX when lock_rx = '0' else '1';

i2c_data_write <= "00000000" when (count_pwr > X"30000" and count_pwr < X"5F50000") else i2c_data_wrr;

process(rx_data)
begin
	   case rx_data is
		when "00000000" => i2c_data_wr <= "00000111"; --MENU
      when "00110001" => i2c_data_wr <= "00001110"; --RAM
		when "00110010" => i2c_data_wr <= "00010101"; --PROG
		when "00110011" => i2c_data_wr <= "00011100"; --IO
		when "00110100" => i2c_data_wr <= "00100011"; --GTX
		when "00110101" => i2c_data_wr <= "00101010"; --SPI
      when others => i2c_data_wr <= "00000111"; --MENU
   end case;
end process;

start_iic <= rx_dv or power_up or five_sec or sec_135 or ten_min or SPI_test_result;

process(start_iic,stop_iic,reset)
begin
  if ((stop_iic = '1') or (reset='0')) then
      count_start <= '0';
  elsif (start_iic = '1') then
		count_start <= '1';
  end if;
end process;

	process(clk,count_start,reset)
	begin
	  if ((count_start='0') or (reset='0')) then
    count_d <= (others => '0');
  elsif (clk'event and clk = '1') then 
			count_d <= count_d + '1';
		end if;
	end process;

stop_iic <= '1' when count_d > X"7FFFFF" and i2c_busy='0' else '0';
ena_i2c_write <= '1' when count_d > X"100" and count_d < X"3000" and count_start='1' else '0';
rw_i2c <= '0' when count_d < X"4000" else '1';
ena_i2c_read <= '1' when count_d > X"5000" else '0';
ena_i2c <= ena_i2c_write or ena_i2c_read;
tx_dv_en <= '1' when count_d > X"6000" else '0';
rst_i2c <= reset;

process(timer)
begin
   case timer(25 downto 23) is 
      when "000" => led0 <= "0001";
		when "001" => led0 <= "0001";
		when "010" => led0 <= "0010";
		when "011" => led0 <= "0100";
		when "100" => led0 <= "1000";
		when "101" => led0 <= "1000";
		when "110" => led0 <= "0100";
		when "111" => led0 <= "0010";
      when others => led0 <= "0000";
   end case;
end process;

TX <= microblaze_tx when ram_mode='1' and count_start='0' else tx_serial;
microblaze_reset <= '1' when ram_mode='1' and count_start='0' else '0';

LED1(0) <= '1' when MPMC_InitDone='1' and timer > X"6317BCA0" and ram_mode = '1' else '0'; --25%
LED1(1) <= '1' when MPMC_InitDone='1' and timer > X"C62F7940" and ram_mode = '1' else '0'; --50%
LED1(2) <= '1' when MPMC_InitDone='1' and timer > X"1294735E0" and ram_mode = '1' else '0'; --75%
LED1(3) <= '1' when MPMC_InitDone='1' and timer > X"18C5EF280" and ram_mode = '1' else '0'; --100%

LED2(0) <= SCL;
LED2(1) <= SCL;
LED2(2) <= SDA;
LED2(3) <= SDA;

i_o <= (J20&J16&J17&J18)&(not io_btn);

io_sum <=
("0000000"&i_o(	0	))
+("0000000"&i_o(	1	))
+("0000000"&i_o(	2	))
+("0000000"&i_o(	3	))
+("0000000"&i_o(	4	))
+("0000000"&i_o(	5	))
+("0000000"&i_o(	6	))
+("0000000"&i_o(	7	))
+("0000000"&i_o(	8	))
+("0000000"&i_o(	9	))
+("0000000"&i_o(	10	))
+("0000000"&i_o(	11	))
+("0000000"&i_o(	12	))
+("0000000"&i_o(	13	))
+("0000000"&i_o(	14	))
+("0000000"&i_o(	15	))
+("0000000"&i_o(	16	))
+("0000000"&i_o(	17	))
+("0000000"&i_o(	18	))
+("0000000"&i_o(	19	))
+("0000000"&i_o(	20	))
+("0000000"&i_o(	21	))
+("0000000"&i_o(	22	))
+("0000000"&i_o(	23	))
+("0000000"&i_o(	24	))
+("0000000"&i_o(	25	))
+("0000000"&i_o(	26	))
+("0000000"&i_o(	27	))
+("0000000"&i_o(	28	))
+("0000000"&i_o(	29	))
+("0000000"&i_o(	30	))
+("0000000"&i_o(	31	))
+("0000000"&i_o(	32	))
+("0000000"&i_o(	33	))
+("0000000"&i_o(	34	))
+("0000000"&i_o(	35	))
+("0000000"&i_o(	36	))
+("0000000"&i_o(	37	))
+("0000000"&i_o(	38	))
+("0000000"&i_o(	39	))
+("0000000"&i_o(	40	))
+("0000000"&i_o(	41	))
+("0000000"&i_o(	42	))
+("0000000"&i_o(	43	))
+("0000000"&i_o(	44	))
+("0000000"&i_o(	45	))
+("0000000"&i_o(	46	))
+("0000000"&i_o(	47	))
+("0000000"&i_o(	48	))
+("0000000"&i_o(	49	))
+("0000000"&i_o(	50	))
+("0000000"&i_o(	51	))
+("0000000"&i_o(	52	))
+("0000000"&i_o(	53	))
+("0000000"&i_o(	54	))
+("0000000"&i_o(	55	))
+("0000000"&i_o(	56	))
+("0000000"&i_o(	57	))
+("0000000"&i_o(	58	))
+("0000000"&i_o(	59	))
+("0000000"&i_o(	60	))
+("0000000"&i_o(	61	))
+("0000000"&i_o(	62	))
+("0000000"&i_o(	63	))
+("0000000"&i_o(	64	))
+("0000000"&i_o(	65	))
+("0000000"&i_o(	66	))
+("0000000"&i_o(	67	))
+("0000000"&i_o(	68	))
+("0000000"&i_o(	69	))
+("0000000"&i_o(	70	))
+("0000000"&i_o(	71	))
+("0000000"&i_o(	72	))
+("0000000"&i_o(	73	))
+("0000000"&i_o(	74	))
+("0000000"&i_o(	75	))
+("0000000"&i_o(	76	))
+("0000000"&i_o(	77	))
+("0000000"&i_o(	78	))
+("0000000"&i_o(	79	))
+("0000000"&i_o(	80	))
+("0000000"&i_o(	81	))
+("0000000"&i_o(	82	))
+("0000000"&i_o(	83	))
+("0000000"&i_o(	84	))
+("0000000"&i_o(	85	))
+("0000000"&i_o(	86	))
+("0000000"&i_o(	87	))
+("0000000"&i_o(	88	))
+("0000000"&i_o(	89	))
+("0000000"&i_o(	90	))
+("0000000"&i_o(	91	))
+("0000000"&i_o(	92	))
+("0000000"&i_o(	93	))
+("0000000"&i_o(	94	))
+("0000000"&i_o(	95	))
+("0000000"&i_o(	96	))
+("0000000"&i_o(	97	))
+("0000000"&i_o(	98	))
+("0000000"&i_o(	99	))
+("0000000"&i_o(	100	))
+("0000000"&i_o(	101	))
+("0000000"&i_o(	102	))
+("0000000"&i_o(	103	))
+("0000000"&i_o(	104	))
+("0000000"&i_o(	105	))
+("0000000"&i_o(	106	))
+("0000000"&i_o(	107	))
+("0000000"&i_o(	108	))
+("0000000"&i_o(	109	))
+("0000000"&i_o(	110	))
+("0000000"&i_o(	111	))
+("0000000"&i_o(	112	))
+("0000000"&i_o(	113	))
+("0000000"&i_o(	114	))
+("0000000"&i_o(	115	))
+("0000000"&i_o(	116	))
+("0000000"&i_o(	117	))
+("0000000"&i_o(	118	))
+("0000000"&i_o(	119	))
+("0000000"&i_o(	120	))
+("0000000"&i_o(	121	))
+("0000000"&i_o(	122	))
+("0000000"&i_o(	123	))
+("0000000"&i_o(	124	))
+("0000000"&i_o(	125	))
+("0000000"&i_o(	126	))
+("0000000"&i_o(	127	))
+("0000000"&i_o(	128	))
+("0000000"&i_o(	129	))
+("0000000"&i_o(	130	))
+("0000000"&i_o(	131	))
+("0000000"&i_o(	132	))
+("0000000"&i_o(	133	))
+("0000000"&i_o(	134	))
+("0000000"&i_o(	135	))
+("0000000"&i_o(	136	))
+("0000000"&i_o(	137	))
+("0000000"&i_o(	138	))
+("0000000"&i_o(	139	))
+("0000000"&i_o(	140	))
+("0000000"&i_o(	141	))
+("0000000"&i_o(	142	))
+("0000000"&i_o(	143	))
+("0000000"&i_o(	144	))
+("0000000"&i_o(	145	))
+("0000000"&i_o(	146	))
+("0000000"&i_o(	147	))
+("0000000"&i_o(	148	))
+("0000000"&i_o(	149	))
+("0000000"&i_o(	150	))
+("0000000"&i_o(	151	))
+("0000000"&i_o(	152	))
+("0000000"&i_o(	153	))
+("0000000"&i_o(	154	))
+("0000000"&i_o(	155	))
+("0000000"&i_o(	156	))
+("0000000"&i_o(	157	))
+("0000000"&i_o(	158	))
+("0000000"&i_o(	159	));

ok <= '1' WHEN io_sum = "00000001" ELSE '0';

led3 <= X"F" when ok='1' else X"0";

SPI_nSS_PIN <= '0' when spi_mode='1' and timer >= X"17D7900" and timer <= X"17E0F00" else '1'; -- 300x SPI CLK
SPI_SCK <= timer(6) when spi_mode='1' and timer >= X"17D7700" and timer <= X"17E0F00" else '1'; --390.625KHz SPI clock
SPI_MOSI_PIN <= '1' when spi_mode='1' and ((timer >= X"17D7900" and timer <= X"17D7980") or (timer >= X"17D7A80" and timer <= X"17D7D00")) else '0'; -- "9F" (READ ID)


--check that flash ID is 20h 20h 17h :

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte1_bit1 <= '0';
 elsif (timer=X"17D7D60" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte1_bit1 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte1_bit2 <= '0';
 elsif (timer=X"17D7DE0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte1_bit2 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte1_bit3 <= '0';
 elsif (timer=X"17D7E60" and SPI_MISO='1' and spi_mode='1') then 
	chk_byte1_bit3 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte1_bit4 <= '0';
 elsif (timer=X"17D7EE0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte1_bit4 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte1_bit5 <= '0';
 elsif (timer=X"17D7F60" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte1_bit5 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte1_bit6 <= '0';
 elsif (timer=X"17D7FE0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte1_bit6 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte1_bit7 <= '0';
 elsif (timer=X"17D8060" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte1_bit7 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte1_bit8 <= '0';
 elsif (timer=X"17D80E0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte1_bit8 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte2_bit1 <= '0';
 elsif (timer=X"17D8160" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte2_bit1 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte2_bit2 <= '0';
 elsif (timer=X"17D81E0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte2_bit2 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte2_bit3 <= '0';
 elsif (timer=X"17D8260" and SPI_MISO='1' and spi_mode='1') then 
	chk_byte2_bit3 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte2_bit4 <= '0';
 elsif (timer=X"17D82E0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte2_bit4 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte2_bit5 <= '0';
 elsif (timer=X"17D8360" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte2_bit5 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte2_bit6 <= '0';
 elsif (timer=X"17D83E0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte2_bit6 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte2_bit7 <= '0';
 elsif (timer=X"17D8460" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte2_bit7 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte2_bit8 <= '0';
 elsif (timer=X"17D84E0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte2_bit8 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte3_bit1 <= '0';
 elsif (timer=X"17D8560" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte3_bit1 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte3_bit2 <= '0';
 elsif (timer=X"17D85E0" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte3_bit2 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte3_bit3 <= '0';
 elsif (timer=X"17D8660" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte3_bit3 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte3_bit4 <= '0';
 elsif (timer=X"17D86E0" and SPI_MISO='1' and spi_mode='1') then 
	chk_byte3_bit4 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte3_bit5 <= '0';
 elsif (timer=X"17D8760" and SPI_MISO='0' and spi_mode='1') then 
	chk_byte3_bit5 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte3_bit6 <= '0';
 elsif (timer=X"17D87E0" and SPI_MISO='1' and spi_mode='1') then 
	chk_byte3_bit6 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte3_bit7 <= '0';
 elsif (timer=X"17D8860" and SPI_MISO='1' and spi_mode='1') then 
	chk_byte3_bit7 <= '1';
	end if;
end process;

process(spi_mode,timer,SPI_MISO)
begin
	if (spi_mode='0') then
   chk_byte3_bit8 <= '0';
 elsif (timer=X"17D88E0" and SPI_MISO='1' and spi_mode='1') then 
	chk_byte3_bit8 <= '1';
	end if;
end process;

SPI_ID_OK <= 
chk_byte1_bit1 and chk_byte1_bit2 and chk_byte1_bit3 and chk_byte1_bit4 and chk_byte1_bit5 and chk_byte1_bit6 and chk_byte1_bit7 and chk_byte1_bit8 and 
chk_byte2_bit1 and chk_byte2_bit2 and chk_byte2_bit3 and chk_byte2_bit4 and chk_byte2_bit5 and chk_byte2_bit6 and chk_byte2_bit7 and chk_byte2_bit8 and 
chk_byte3_bit1 and chk_byte3_bit2 and chk_byte3_bit3 and chk_byte3_bit4 and chk_byte3_bit5 and chk_byte3_bit6 and chk_byte3_bit7 and chk_byte3_bit8;

process(timer,SPI_ID_OK)
begin
if (SPI_ID_OK = '0') then led5 <= "0000";
else
   case timer(24) is 
      when '0' => led5 <= "0110";
		when '1' => led5 <= "1001";
      when others => led5 <= "0000";
   end case;
	end if;
end process;

SPI_test_result <= '1' when spi_mode='1' and timer = X"2001111" else '0';

i2c_data_wrr <= "00110001" when (SPI_ID_OK = '1' and timer > X"2000000" and timer < X"2002222" and spi_mode='1') else
"00111000" when (SPI_ID_OK = '0' and timer > X"2000000" and timer < X"2002222" and spi_mode='1') else i2c_data_wr;

end Behavioral;